/**
 * Copyright 2020 Huawei Technologies Co., Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef MINDSPORE_CORE_OPS_OP_UTILS_H
#define MINDSPORE_CORE_OPS_OP_UTILS_H
#include <string>
#include <set>
#include <vector>
#include <algorithm>
#include <memory>
#include "abstract/primitive_infer_map.h"
#include "utils/check_convert_utils.h"

namespace mindspore::ops {
constexpr auto kAlpha = "alpha";
constexpr auto kActivation = "activation";
constexpr auto kActivationType = "activation_type";
constexpr auto kAttentionQActType = "attention_q_act_type";
constexpr auto kAttentionKActType = "attention_k_act_type";
constexpr auto kAttentionVActType = "attention_v_act_type";
constexpr auto kAddress = "address";
constexpr auto kAlignCorners = "align_corners";
constexpr auto kAttr = "attr";
constexpr auto kAspectRatios = "aspect_ratios";
constexpr auto kAxes = "axes";
constexpr auto kAxis = "axis";
constexpr auto kAxisType = "axis_type";
constexpr auto kBaseSize = "base_size";
constexpr auto kBatchDim = "batch_dim";
constexpr auto kBeginMask = "begin_mask";
constexpr auto kBeginNormAxis = "begin_norm_axis";
constexpr auto kBeginParamsAxis = "begin_params_axis";
constexpr auto kBeta = "beta";
constexpr auto kBias = "bias";
constexpr auto kBidirectional = "bidirectional";
constexpr auto kBlockSize = "block_size";
constexpr auto kBlockShape = "block_shape";
constexpr auto kCellClip = "cell_clip";
constexpr auto kCellDepth = "cell_depth";
constexpr auto kCenterPointBox = "center_point_box";
constexpr auto kClip = "clip";
constexpr auto kCondition = "condition";
constexpr auto kCrops = "crops";
constexpr auto kCustom = "custom";
constexpr auto kDampening = "dampening";
constexpr auto kDataType = "data_type";
constexpr auto kDctCoeffNum = "dct_coeff_num";
constexpr auto kDelta = "delta";
constexpr auto kDependMode = "depend_mode";
constexpr auto kDepthRadius = "depth_radius";
constexpr auto kDetectionsPerClass = "detections_per_class";
constexpr auto kDilation = "dilation";
constexpr auto kDropout = "dropout";
constexpr auto kDstT = "dst_t";
constexpr auto kDType = "d_type";
constexpr auto kEllipsisMask = "ellipsis_mask";
constexpr auto kEndMask = "end_mask";
constexpr auto kEps = "eps";
constexpr auto kEpsilon = "epsilon";
constexpr auto kElement_dtype = "element_dtype";
constexpr auto kFeatStride = "feat_stride";
constexpr auto kFftLength = "fft_length";
constexpr auto kFilterBankChannelNum = "filter_bank_channel_num";
constexpr auto kFlip = "flip";
constexpr auto kFormat = "format";
constexpr auto kOriginalFormat = "OriginalFormat";
constexpr auto kFreqLowerLimit = "freq_lower_limit";
constexpr auto kFreqUpperLimit = "freq_upper_limit";
constexpr auto kFreezeBn = "freeze_bn";
constexpr auto kGateOrder = "gate_order";
constexpr auto kGlobal = "global";
constexpr auto kGrad = "grad";
constexpr auto kIsGrad = "is_grad";
constexpr auto kGradientScale = "gradient_scale";
constexpr auto kGradX = "grad_x";
constexpr auto kGradY = "grad_y";
constexpr auto kGroup = "group";
constexpr auto kHasBias = "has_bias";
constexpr auto kAttentionHasMask = "attention_has_mask";
constexpr auto kHiddenSize = "hidden_size";
constexpr auto kId = "id";
constexpr auto kImageSizeH = "image_size_h";
constexpr auto kImageSizeW = "image_size_w";
constexpr auto kIncludeALLGrams = "include_all_grams";
constexpr auto kInputSize = "input_size";
constexpr auto kInChannel = "in_channel";
constexpr auto kInputShape = "input_shape";
constexpr auto kIoFormat = "io_format";
constexpr auto kIsScale = "is_scale";
constexpr auto kIsTraining = "is_training";
constexpr auto kKeepDims = "keep_dims";
constexpr auto kKeepProb = "keep_prob";
constexpr auto kKernelSize = "kernel_size";
constexpr auto kLimit = "limit";
constexpr auto kMagSquare = "mag_square";
constexpr auto kMax = "max";
constexpr auto kMaxSizes = "max_sizes";
constexpr auto kMaxSkipSize = "max_skip_size";
constexpr auto kMaxClassesPerDetection = "max_classes_per_detection";
constexpr auto kMaxDetections = "max_detections";
constexpr auto kMaxNorm = "max_norm";
constexpr auto kMin = "min";
constexpr auto kMinSize = "min_size";
constexpr auto kMinSizes = "min_sizes";
constexpr auto kMode = "mode";
constexpr auto kMomentum = "momentum";
constexpr auto kN = "n";
constexpr auto kNarrowRange = "narrow_range";
constexpr auto kNesterov = "nesterov";
constexpr auto kNewAxisMask = "new_axis_mask";
constexpr auto kNgramSize = "ngram_size";
constexpr auto kNmsThresh = "nms_thresh";
constexpr auto kNormRegion = "norm_region";
constexpr auto kNumLayers = "num_layers";
constexpr auto kNumElements = "num_elements";
constexpr auto kNumBits = "num_bits";
constexpr auto kNumDirections = "num_directions";
constexpr auto kNumProj = "num_proj";
constexpr auto kAttentionNumHeads = "attention_num_heads";
constexpr auto kAttentionSizePerHead = "attention_size_per_head";
constexpr auto kAttentionFromSeqLen = "attention_from_seq_len";
constexpr auto kAttentionToSeqLen = "attention_to_seq_len";
constexpr auto kOffset = "offset";
constexpr auto kNmsIouThreshold = "nms_iou_threshold";
constexpr auto kNmsScoreThreshold = "nms_score_threshold";
constexpr auto kNumClasses = "num_classes";
constexpr auto kOffsets = "offsets";
constexpr auto kOffsetA = "offset_a";
constexpr auto kOrder = "order";
constexpr auto kOutChannel = "out_channel";
constexpr auto kOutMaxValue = "out_max_value";
constexpr auto kOutputChannel = "output_channel";
constexpr auto kOutputNum = "output_num";
constexpr auto kOutputPaddings = "output_paddings";
constexpr auto kOutputType = "output_type";
constexpr auto kOutQuantized = "out_quantized";
constexpr auto kP = "p";
constexpr auto kPad = "pad";
constexpr auto kPadding = "padding";
constexpr auto kPaddingsElementSize = "paddings_element_size";
constexpr auto kPaddingsSize = "paddings_size";
constexpr auto kPadItem = "pad_item";
constexpr auto kPadList = "pad_list";
constexpr auto kPadMode = "pad_mode";
constexpr auto kPads = "pads";
constexpr auto kPadSize = "pad_size";
constexpr auto kPooledH = "pooled_h";
constexpr auto kPooledW = "pooled_w";
constexpr auto kPoolMode = "pool_mode";
constexpr auto kCeilMode = "ceil_mode";
constexpr auto kCountIncludePad = "count_include_pad";
constexpr auto kDivisorOverride = "divisor_override";
constexpr auto kPostNmsTopn = "post_nms_topn";
constexpr auto kPower = "power";
constexpr auto kPreNmsTopn = "pre_nms_topn";
constexpr auto kRatio = "ratio";
constexpr auto kReduction = "reduction";
constexpr auto kRootRank = "root_rank";
constexpr auto kRoundMode = "round_mode";
constexpr auto kSame = "same";
constexpr auto kScale = "scale";
constexpr auto kSeed = "seed";
constexpr auto kSeed2 = "seed2";
constexpr auto kSeqDim = "seq_dim";
constexpr auto kSetattrFlag = "setattr_flag";
constexpr auto kShape = "shape";
constexpr auto kShapeGamma = "shape_gamma";
constexpr auto kShapeSize = "shape_size";
constexpr auto kShift = "shift";
constexpr auto kShrinkAxisMask = "shrink_axis_mask";
constexpr auto kSize = "size";
constexpr auto kSorted = "sorted";
constexpr auto kSrcT = "src_t";
constexpr auto kStart = "start";
constexpr auto kStepH = "step_h";
constexpr auto kStepW = "step_w";
constexpr auto kStride = "stride";
constexpr auto kStrides = "strides";
constexpr auto kShapeType = "shape_type";
constexpr auto kSubGraphIndex = "sub_graph_index";
constexpr auto kSummarize = "summarize";
constexpr auto kTimeMajor = "time_major";
constexpr auto kTopK = "top_k";
constexpr auto kTransposeA = "transpose_a";
constexpr auto kTransposeB = "transpose_b";
constexpr auto kNegativeSlope = "negative_slope";
constexpr auto kType = "type";
constexpr auto kUseAxis = "use_axis";
constexpr auto kUseLocking = "use_locking";
constexpr auto kUseNesterov = "use_nesterov";
constexpr auto kUseNesteroy = "use_nesteroy";
constexpr auto kUseRegularNms = "use_regular_nms";
constexpr auto kValid = "valid";
constexpr auto kValue = "value";
constexpr auto kVariances = "variances";
constexpr auto kWeightDecay = "weight_decay";
constexpr auto kWeightThreshold = "weight_threshold";
constexpr auto kWindow = "window";
constexpr auto kWindowSize = "window_size";
constexpr auto kPaddings = "paddings";
constexpr auto kInput_size = "input_size";
constexpr auto kHidden_size = "hidden_size";
constexpr auto kChannelShared = "channel_shared";
constexpr auto kSlope = "slope";
constexpr auto kBase = "base";
constexpr auto kConstantValue = "constant_value";
constexpr auto kSizeSplits = "size_splits";
constexpr auto kDims = "dims";
constexpr auto kPaddingMode = "padding_mode";
constexpr auto kLargest = "largest";
constexpr auto kElementwiseAffine = "elementwise_affine";
constexpr auto kMinVal = "min_val";
constexpr auto kMaxVal = "max_val";
constexpr auto kMethod = "method";
constexpr auto kNewHeight = "new_height";
constexpr auto kNewWidth = "new_width";
constexpr auto kPreserveAspectRatio = "preserve_aspect_ratio";
constexpr auto kCoordinateTransformMode = "coordinate_transform_mode";
constexpr auto kCubicCoeff = "cubic_coeff";
constexpr auto kExcludeOutside = "exclude_outside";
constexpr auto kExtrapolationValue = "extrapolation_value";
constexpr auto kNearestMode = "nearest_mode";
constexpr auto kReduceToEnd = "reduce_to_end";
constexpr auto kResetAfter = "reset_after";
constexpr auto kCoeff = "coeff";
constexpr auto kIsDepthWise = "is_depth_wise";
constexpr auto kZoneoutCell = "zoneout_cell";
constexpr auto kZoneoutHidden = "zoneout_hidden";
constexpr auto kSpliceContext = "context";
constexpr auto kSpliceForwardIndexes = "forward_indexes";
constexpr auto kSpliceOutputDims = "output_dim";
constexpr auto kSideEffectIO = "side_effect_io";
constexpr auto kDeviceType = "device_type";
constexpr auto kExclusive = "exclusive";
constexpr auto kReverse = "reverse";
constexpr auto kSplitStride = "split_stride";
constexpr auto kExtendTop = "extend_top";
constexpr auto kExtendBottom = "extend_bottom";
constexpr auto kNumberSplit = "number_split";
constexpr auto kSplitDim = "split_dim";
constexpr auto kPadTop = "pad_top";
constexpr auto kTransFormat = "trans_format";
constexpr auto kApproximate = "approximate";
constexpr auto kNumOutput = "num_output";
constexpr auto kUseGlobalStats = "use_global_stats";
constexpr auto kFmkType = "fmk_type";

enum Index : size_t {
  kInputIndex0 = 0,
  kInputIndex1,
  kInputIndex2,
  kInputIndex3,
  kInputIndex4,
  kInputIndex5,
  kInputIndex6,
  kInputIndex7,
  kInputIndex8,
  kInputIndex9,
  kInputIndex10,
  kInputIndex11,
  kInputIndex12,
  kInputIndex13,
  kInputIndex14,
  kInputIndex15,
  kInputIndex16,
};

const std::set<TypePtr> common_valid_types = {kInt8,   kInt16,  kInt32,   kInt64,   kUInt8,  kUInt16,
                                              kUInt32, kUInt64, kFloat16, kFloat32, kFloat64};

const std::set<TypePtr> all_types = {
  kBool,   kInt,    kInt8,   kInt16, kInt32,   kInt64,   kUInt,    kUInt8,
  kUInt16, kUInt32, kUInt64, kFloat, kFloat16, kFloat32, kFloat64, kComplex64,
};

std::vector<int64_t> CalBroadCastShape(std::vector<int64_t> x_shape, std::vector<int64_t> y_shape,
                                       const std::string &op_name, const std::string &op_x_name = "input1",
                                       const std::string &op_y_name = "input2");
abstract::ShapePtr BroadCastInferShape(const std::string &op_name, const std::vector<AbstractBasePtr> &input_args);
}  // namespace mindspore::ops
#endif  // MINDSPORE_CORE_OPS_OP_UTILS_H
